<?php

namespace Drupal\photos\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\Node;
use Drupal\photos\PhotosAlbum;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Edit images and image details.
 */
class PhotosEditController {

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   */
  public function access(AccountInterface $account) {
    // Check for available parameters.
    $nid = \Drupal::routeMatch()->getParameter('node');
    $fid = \Drupal::routeMatch()->getParameter('file');
    if ($nid && $fid) {
      // Update cover.
      $node = Node::load($nid);
      if (_photos_access('editAlbum', $node)) {
        // Allowed to update album cover image.
        return AccessResult::allowed();
      }
      else {
        // Deny access.
        return AccessResult::forbidden();
      }
    }
    elseif ($fid) {
      // Check if edit or delete.
      $current_path = \Drupal::service('path.current')->getPath();
      $path_args = explode('/', $current_path);
      if (isset($path_args[4])) {
        if ($path_args[4] == 'edit') {
          if (_photos_access('imageEdit', $fid)) {
            // Allowed to edit image.
            return AccessResult::allowed();
          }
        }
        elseif ($path_args[4] == 'delete') {
          if (_photos_access('imageDelete', $fid)) {
            // Allowed to delete image.
            return AccessResult::allowed();
          }
        }
      }
      // Deny access.
      return AccessResult::forbidden();
    }
    else {
      return AccessResult::neutral();
    }
  }

  /**
   * Edit image.
   */
  public function editImage($file) {
    $fid = $file;
    $query = db_select('file_managed', 'f');
    $query->join('photos_image', 'p', 'p.fid = f.fid');
    $query->join('users_field_data', 'u', 'f.uid = u.uid');
    $query->fields('f', ['uri', 'filemime', 'created', 'filename', 'filesize']);
    $query->fields('p');
    $query->fields('u', ['uid', 'name']);
    $query->condition('f.fid', $fid);
    $image = $query->execute()->fetchObject();

    if ($image && isset($image->fid)) {
      $edit_form = \Drupal::formBuilder()->getForm('\Drupal\photos\Form\PhotosImageEditForm', $image);
      return $edit_form;
    }
    else {
      throw new NotFoundHttpException();
    }

  }

  /**
   * Delete image from gallery and site.
   */
  public function deleteImage($file) {
    $fid = $file;
    if (\Drupal::moduleHandler()->moduleExists('colorbox')) {
      // Dispaly form in modal popup.
      // @todo does this still work?
      $confirm_delete_form = \Drupal::formBuilder()->getForm('\Drupal\photos\Form\PhotosImageConfirmDeleteForm', $fid);
      print \Drupal::service("renderer")->render($confirm_delete_form);
    }
    else {
      // Render full page.
      return \Drupal::formBuilder()->getForm('\Drupal\photos\Form\PhotosImageConfirmDeleteForm', $fid);
    }
  }

  /**
   * Ajax edit image.
   */
  public function ajaxEditUpdate($fid = NULL) {
    $message = '';
    if (isset($_POST['id'])) {
      $value = isset($_POST['value']) ? trim($_POST['value']) : '';
      $id = Html::escape($_POST['id']);
      // Get fid.
      if (strstr($id, 'title')) {
        $switch = 'title';
        $fid = str_replace('photos-image-edit-title-', '', $id);
      }
      elseif (strstr($id, 'des')) {
        $switch = 'des';
        $fid = str_replace('photos-image-edit-des-', '', $id);
      }
      $fid = filter_var($fid, FILTER_SANITIZE_NUMBER_INT);
      // Check user image edit permissions.
      // @todo photos.routing.yml _csrf_token: 'TRUE'.
      if ($fid && _photos_access('imageEdit', $fid)) {
        switch ($switch) {
          case 'title':
            db_update('photos_image')
              ->fields([
                'title' => $value,
              ])
              ->condition('fid', $fid)
              ->execute();
            $message = Html::escape($value);
            break;

          case 'des':
            db_update('photos_image')
              ->fields([
                'des' => $value,
              ])
              ->condition('fid', $fid)
              ->execute();
            $message = Html::escape($value);
            break;
        }
        // Clear cache.
        $pid = db_query("SELECT pid FROM {photos_image} WHERE fid = :fid", [':fid' => $fid])->fetchField();
        if ($pid) {
          Cache::invalidateTags(['node:' . $pid, 'photos:album:' . $pid]);
        }
        Cache::invalidateTags(['photos:image:' . $fid]);
      }
    }

    // Build plain text response.
    $response = new Response();
    $response->headers->set('Content-Type', 'text/plain');
    $response->setContent($message);
    return $response;
  }

  /**
   * Ajax edit image load text.
   */
  public function ajaxEditUpdateLoad() {
    $message = '';
    if (isset($_POST['id'])) {
      $id = Html::escape($_POST['id']);
      if (strstr($id, 'title')) {
        $switch = 'title';
        $fid = str_replace('photos-image-edit-title-', '', $id);
      }
      elseif (strstr($id, 'des')) {
        $switch = 'des';
        $fid = str_replace('photos-image-edit-des-', '', $id);
      }
      $fid = filter_var($fid, FILTER_SANITIZE_NUMBER_INT);
      // Check user image edit permissions.
      // @todo photos.routing.yml _csrf_token: 'TRUE'.
      if ($fid && _photos_access('imageEdit', $fid)) {
        switch ($switch) {
          case 'title':
            $value = db_query("SELECT title FROM {photos_image} WHERE fid = :fid", [':fid' => $fid])->fetchField();
            $message = $value;
            break;

          case 'des':
            $value = db_query("SELECT des FROM {photos_image} WHERE fid = :fid", [':fid' => $fid])->fetchField();
            $message = $value;
            break;
        }
        // Clear cache.
        $pid = db_query("SELECT pid FROM {photos_image} WHERE fid = :fid", [':fid' => $fid])->fetchField();
        if ($pid) {
          Cache::invalidateTags(['node:' . $pid, 'photos:album:' . $pid]);
        }
        Cache::invalidateTags(['photos:image:' . $fid]);
      }
    }

    // Build plain text response.
    $response = new Response();
    $response->headers->set('Content-Type', 'text/plain');
    $response->setContent($message);
    return $response;
  }

  /**
   * Set album cover.
   */
  public function setAlbumCover($node, $file) {
    $pid = db_query('SELECT pid FROM {photos_image} WHERE fid = :fid', [':fid' => $file])->fetchField();
    if ($pid == $node) {
      $album = new PhotosAlbum($pid);
      $album->setCover($file);
      $goto = isset($_GET['destination']) ? $_GET['destination'] : 'photos/album/' . $node;
      $goto = Url::fromUri('base:' . $goto)->toString();
      $response = new RedirectResponse($goto);
      $response->send();
      return [
        '#markup' => t('Cover successfully set.'),
      ];
    }
    else {
      throw new NotFoundHttpException();
    }
  }

}
