<?php

namespace Drupal\photos\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Url;
use Drupal\photos\PhotosImage;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Image view controller.
 */
class PhotosImageController {

  /**
   * Set page title.
   */
  public function getTitle() {
    // Get node object.
    $fid = \Drupal::routeMatch()->getParameter('file');
    $title = db_query("SELECT title FROM {photos_image} WHERE fid = :fid", [':fid' => $fid])->fetchField();
    return $title;
  }

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   */
  public function access(AccountInterface $account) {
    // Check if user can view account photos.
    $fid = \Drupal::routeMatch()->getParameter('file');
    if (_photos_access('imageView', $fid)) {
      // Allow access.
      return AccessResult::allowed();
    }
    else {
      return AccessResult::forbidden();
    }
  }

  /**
   * Returns content for single image.
   *
   * @return string
   *   A HTML-formatted string with the administrative page content.
   */
  public function contentOverview() {
    $fid = \Drupal::routeMatch()->getParameter('file');
    if (!is_numeric($fid)) {
      throw new NotFoundHttpException();
    }
    $photos_image = new PhotosImage($fid);
    $image = $photos_image->load();

    if (!$image) {
      throw new NotFoundHttpException();
    }

    $node = \Drupal::entityManager()->getStorage('node')->load($image->pid);
    if (_photos_access('imageEdit', $node)) {
      $image->ajax['edit_url'] = Url::fromUri('base:photos/image/' . $image->fid . '/update')->toString();
      // Set album cover.
      $url = Url::fromRoute('photos.album.update.cover', ['node' => $image->pid, 'file' => $fid]);
      $image->links['cover'] = \Drupal::l(t('Set to Cover'), $url, [
        'query' => drupal_get_destination(),
      ]);
    }
    $image->class = [
      'title_class' => '',
      'des_class' => '',
    ];
    $image->id = [
      'des_edit' => '',
      'title_edit' => '',
    ];
    $edit = _photos_access('imageEdit', $node);
    if ($edit) {
      // Image edit link.
      $url = Url::fromUri('base:photos/image/' . $image->fid . '/edit');
      $image->ajax['edit_link'] = \Drupal::l(t('Edit'), $url, [
        'query' => [
          'destination' => 'photos/image/' . $image->fid,
        ],
        'attributes' => [
          'class' => ['colorbox-load', 'photos-edit-edit'],
        ],
      ]);

      $image->class = [
        'title_class' => ' jQueryeditable_edit_title',
        'des_class' => ' jQueryeditable_edit_des',
      ];
      $image->id = [
        'des_edit' => ' id="photos-image-edit-des-' . $image->fid . '"',
        'title_edit' => ' id="photos-image-edit-title-' . $image->fid . '"',
      ];
      $jeditable_library = \Drupal::service('library.discovery')->getLibraryByName('photos', 'photos.jeditable');
    }
    if (_photos_access('imageDelete', $node)) {
      // Image delete link.
      $url = Url::fromUri('base:photos/image/' . $image->fid . '/delete');
      $image->ajax['del_link'] = \Drupal::l(t('Delete'), $url, [
        'query' => [
          'destination' => 'node/' . $image->pid,
        ],
        'attributes' => [
          'class' => ['colorbox-load', 'photos-edit-delete'],
        ],
      ]);
    }
    if (\Drupal::config('photos.settings')->get('photos_comment')) {
      // Comment integration.
      $render_comment = [
        '#theme' => 'photos_comment_count',
        '#comcount' => $image->comcount,
      ];
      $image->links['comment'] = $render_comment;
    }

    // Album images.
    $pager_type = 'pid';
    $pager_id = $image->pid;
    $data = isset($image->data) ? unserialize($image->data) : [];
    $style_name = isset($data['view_imagesize']) ? $data['view_imagesize'] : \Drupal::config('photos.settings')->get('photos_display_view_imagesize');

    $image->links['pager'] = $photos_image->pager($pager_id, $pager_type);
    $image->view = [
      '#theme' => 'photos_image_html',
      '#style_name' => $style_name,
      '#image' => $image,
      '#cache' => [
        'tags' => [
          'photos:image:' . $fid,
        ],
      ],
    ];

    // Get comments.
    $image->comment['view'] = $photos_image->comments($image->comcount, $node);
    // Check count image views variable.
    $photos_image_count = \Drupal::config('photos.settings')->get('photos_image_count');
    $image->disable_photos_image_count = $photos_image_count;
    if (!$photos_image_count) {
      $count = 1;
      db_update('photos_image')
        ->fields(['count' => $count])
        ->expression('count', 'count + :count', [':count' => $count])
        ->condition('fid', $fid)
        ->execute();
    }
    $image->title = Html::escape($image->title);
    $image->des = Html::escape($image->des);

    $GLOBALS['photos'][$image->fid . '_pid'] = $image->pid;

    $image_view = [
      '#theme' => 'photos_image_view',
      '#image' => $image,
      '#display_type' => 'view',
      '#cache' => [
        'tags' => [
          'photos:image:' . $fid,
        ],
      ],
    ];
    // Check for Jeditable library.
    // @todo move to static public function?
    if ($edit && isset($jeditable_library['js']) && file_exists($jeditable_library['js'][0]['data'])) {
      $image_view['#attached']['library'][] = 'photos/photos.jeditable';
    }

    return $image_view;
  }

}
