<?php

namespace Drupal\photos\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\photos\PhotosAlbum;
use Drupal\photos\PhotosImage;

/**
 * Controller to display recent images.
 */
class PhotosImagesRecentController {

  /**
   * Returns content for recent images.
   *
   * @return array
   *   An array containing markup for the page content.
   */
  public function contentOverview() {
    $build = [];
    // Prepare query.
    $order = explode('|', \Drupal::config('photos.settings')->get('photos_display_imageorder'));
    $order = PhotosAlbum::orderValueChange($order[0], $order[1]);
    // Recent images default sort should be created desc.
    $column = (isset($_GET['field'])) ? Html::escape($_GET['field']) : 'created';
    $sort = (isset($_GET['sort'])) ? Html::escape($_GET['sort']) : 'desc';
    $term = PhotosAlbum::orderValue($column, $sort, \Drupal::config('photos.settings')->get('photos_display_viewpager'), $order);
    // Query recent images.
    $query = db_select('file_managed', 'f')
      ->extend('Drupal\Core\Database\Query\PagerSelectExtender');
    $query->join('photos_image', 'p', 'p.fid = f.fid');
    $query->join('node_field_data', 'n', 'n.nid = p.pid');
    $query->join('users_field_data', 'u', 'u.uid = f.uid');
    $query->fields('f', ['fid']);
    $query->orderBy($term['order']['column'], $term['order']['sort']);
    $query->limit($term['limit']);
    $query->addTag('node_access');
    $results = $query->execute();

    // Image count.
    $album['links'] = PhotosAlbum::orderLinks('photos/image', PhotosAlbum::getCount('site_image'), 0, 1);
    $com = \Drupal::config('photos.settings')->get('photos_comment');

    // @todo check permission to edit.
    $edit = 0;

    $style_name = \Drupal::config('photos.settings')->get('photos_display_list_imagesize');
    foreach ($results as $result) {
      $photos_image = new PhotosImage($result->fid);
      $image = $photos_image->load();
      $image->title = Html::escape($image->title);
      $image->des = Html::escape($image->des);
      $image->view = [
        '#theme' => 'photos_image_html',
        '#style_name' => $style_name,
        '#image' => $image,
      ];

      $image->url = Url::fromUri('base:photos/image/' . $image->fid)->toString();

      if ($com) {
        $image->links['comment'] = [
          '#theme' => 'photos_comment_count',
          '#comcount' => $image->comcount,
          '#url' => $image->url,
        ];
      }
      // Check count image views variable.
      $photos_image_count = \Drupal::config('photos.settings')->get('photos_image_count');
      if (!$photos_image_count && isset($image->count)) {
        $image->links['count'] = \Drupal::translation()->formatPlural($image->count, '@cou visit', '@cou visits', ['@cou' => $image->count]);
      }
      // Get username.
      $name = '';
      if (!empty($image->uid)) {
        $account = \Drupal::entityManager()->getStorage('user')->load($image->uid);
        $name_render_array = [
          '#theme' => 'username',
          '#account' => $account,
        ];
        $name = drupal_render($name_render_array);
      }
      $image->links['info'] = t('Uploaded on @time by @name in @title', [
        '@name' => $name,
        '@time' => \Drupal::service('date.formatter')->format($image->created, 'short'),
        '@title' => \Drupal::l($image->node_title, Url::fromUri('base:photos/album/' . $image->pid)),
      ]);
      $image->class = [
        'title_class' => '',
        'des_class' => '',
      ];
      $image->id = [
        'des_edit' => '',
        'title_edit' => '',
      ];
      $image->ajax['del_id'] = '';
      if ($edit) {
        $image->ajax['edit_url'] = $image->url . '/update';
        $current_path = \Drupal::service('path.current')->getPath();
        $image->ajax['edit_link'] = \Drupal::l(t('Edit'), Url::fromUri('base:photos/image/' . $image->fid . '/edit'), [
          'query' => [
            'destination' => $current_path,
            'pid' => $image->pid,
            'uid' => $image->uid,
          ],
          'attributes' => [
            'class' => ['colorbox-load', 'photos-edit-edit'],
          ],
        ]);

        $image->class = [
          'title_class' => ' jQueryeditable_edit_title',
          'des_class' => ' jQueryeditable_edit_des',
        ];
        $image->id = [
          'des_edit' => ' id="photos-image-edit-des-' . $image->fid . '"',
          'title_edit' => ' id="photos-image-edit-title-' . $image->fid . '"',
        ];
        $image->ajax['del_id'] = 'id="photos_ajax_del_' . $image->fid . '"';
        $image->ajax['del_link'] = \Drupal::l(t('Delete'), Url::fromUri('base:photos/image/' . $image->fid . '/delete'), [
          'query' => [
            'destination' => $current_path,
            'pid' => $image->pid,
            'uid' => $image->uid,
          ],
          'attributes' => [
            'class' => ['photos-edit-delete'],
            'alt' => 'photos_ajax_del_' . $image->fid,
          ],
        ]);

        // Set cover link.
        $url_query = drupal_get_destination();
        $cover_url = Url::fromRoute('photos.album.update.cover', [
          'node' => $image->pid,
          'file' => $image->fid,
        ],
        [
          'query' => $url_query,
        ]);
        $image->links['cover'] = \Drupal::l(t('Set to Cover'), $cover_url);
      }
      $album['view'][] = [
        '#theme' => 'photos_image_view',
        '#image' => $image,
        '#display_type' => 'list',
      ];
    }
    if (isset($album['view'][0])) {
      // Set pager.
      $album['pager'] = ['#type' => 'pager'];
      $render_album_view = [
        '#theme' => 'photos_album_view',
        '#album' => $album,
        '#node' => NULL,
      ];
      $content = drupal_render($render_album_view);
    }
    else {
      if ($account <> FALSE) {
        $content = t('@name has not uploaded any images yet.', ['@name' => $account->name]);
      }
      else {
        $content = t('No images have been uploaded yet.');
      }
    }
    $build = [
      '#markup' => $content,
    ];

    return $build;
  }

}
